<!DOCTYPE html>
<html lang="en">
<head>
    {% include 'core/head.html' %}
    {% include 'core/scripts.html' %}
</head>
<body>
    
    {% block content %}{% endblock %}
    <footer>
        <div class="version">
            v{{app_version}}
            {% if update_available %} 
                <span>(Update Available)</span> 
            {% endif %}
        </div>
        
    </footer>
</body>
</html>
